@@ -34,6 +34,8 @@ static void MIX_AREAS1(unsigned int size,
 		       volatile signed int *sum, size_t dst_step,
 		       size_t src_step, size_t sum_step)
 {
+	unsigned int old_ebx;
+
 	/*
 	 *  ESI - src
 	 *  EDI - dst
@@ -45,14 +47,16 @@ static void MIX_AREAS1(unsigned int size,
 	__asm__ __volatile__ (
 		"\n"
 
-		"\tpush %%ebx\n"	/* ebx is GOT pointer (-fPIC) */
+		"\tmovl %%ebx, %7\n"	/* ebx is GOT pointer (-fPIC) */
 		/*
 		 *  initialization, load ESI, EDI, EBX registers
 		 */
 		"\tmovl %1, %%edi\n"
 		"\tmovl %2, %%esi\n"
 		"\tmovl %3, %%ebx\n"
-		"\tjmp 2f\n"
+		"\tcmpl $0, %0\n"
+		"\tjnz 2f\n"
+		"\tjmp 7f\n"
 
 
 		/*
@@ -73,9 +77,8 @@ static void MIX_AREAS1(unsigned int size,
 		 */
 
 		"2:"
-		"\txor %%ax, %%ax\n"
-		"\tmovw %%ax, %%cx\n"
-		"\tincw  %%cx\n"
+		"\tmovw $0, %%ax\n"
+		"\tmovw $1, %%cx\n"
 		"\tmovl (%%ebx), %%edx\n"
 		"\t" LOCK_PREFIX "cmpxchgw %%cx, (%%edi)\n"
 		"\tmovswl (%%esi), %%ecx\n"
@@ -106,8 +109,8 @@ static void MIX_AREAS1(unsigned int size,
 		 * while (size-- > 0)
 		 */
 		"\tdecl %0\n"
-		"\tjz  7f\n"
-		"\tjmp 1b\n"
+		"\tjnz 1b\n"
+		"\tjmp 7f\n"
 
 		/*
 		 *  sample > 0x7fff
@@ -137,10 +140,12 @@ static void MIX_AREAS1(unsigned int size,
 		"\tjnz 1b\n"
 		
 		"7:"
-		"\tpop %%ebx\n"		/* ebx is GOT pointer (-fPIC) */
+		"\tmovl %7, %%ebx\n"	/* ebx is GOT pointer (-fPIC) */
 
 		: /* no output regs */
-		: "m" (size), "m" (dst), "m" (src), "m" (sum), "m" (dst_step), "m" (src_step), "m" (sum_step)
+		: "m" (size), "m" (dst), "m" (src),
+		  "m" (sum), "m" (dst_step), "m" (src_step),
+		  "m" (sum_step), "m" (old_ebx)
 		: "esi", "edi", "edx", "ecx", "eax"
 	);
 }
@@ -153,6 +158,8 @@ static void MIX_AREAS1_MMX(unsigned int size,
 			   volatile signed int *sum, size_t dst_step,
 			   size_t src_step, size_t sum_step)
 {
+	unsigned int old_ebx;
+
 	/*
 	 *  ESI - src
 	 *  EDI - dst
@@ -164,14 +171,16 @@ static void MIX_AREAS1_MMX(unsigned int size,
 	__asm__ __volatile__ (
 		"\n"
 
-		"\tpush %%ebx\n"	/* ebx is GOT pointer (-fPIC) */
+		"\tmovl %%ebx, %7\n"	/* ebx is GOT pointer (-fPIC) */
 		/*
 		 *  initialization, load ESI, EDI, EBX registers
 		 */
 		"\tmovl %1, %%edi\n"
 		"\tmovl %2, %%esi\n"
 		"\tmovl %3, %%ebx\n"
-		"\tjmp  2f\n"
+		"\tcmpl $0, %0\n"
+		"\tjnz 2f\n"
+		"\tjmp 5f\n"
 
 		"\t.p2align 4,,15\n"
 		"1:"
@@ -187,9 +196,8 @@ static void MIX_AREAS1_MMX(unsigned int size,
 		 *     sample -= sum_sample;
 		 *   xadd(*sum, sample);
 		 */
-		"\txor  %%ax, %%ax\n"
-		"\tmovw %%ax, %%cx\n"
-		"\tincw %%cx\n"
+		"\tmovw $0, %%ax\n"
+		"\tmovw $1, %%cx\n"
 		"\tmovl (%%ebx), %%edx\n"
 		"\t" LOCK_PREFIX "cmpxchgw %%cx, (%%edi)\n"
 		"\tmovswl (%%esi), %%ecx\n"
@@ -221,10 +229,13 @@ static void MIX_AREAS1_MMX(unsigned int size,
 		"\tdecl %0\n"
 		"\tjnz 1b\n"
 		"\temms\n"
-		"\tpop %%ebx\n"		/* ebx is GOT pointer (-fPIC) */
+                "5:"
+		"\tmovl %7, %%ebx\n"	/* ebx is GOT pointer (-fPIC) */
 
 		: /* no output regs */
-		: "m" (size), "m" (dst), "m" (src), "m" (sum), "m" (dst_step), "m" (src_step), "m" (sum_step)
+		: "m" (size), "m" (dst), "m" (src),
+		  "m" (sum), "m" (dst_step), "m" (src_step),
+		  "m" (sum_step), "m" (old_ebx)
 		: "esi", "edi", "edx", "ecx", "eax"
 	);
 }
@@ -237,6 +248,8 @@ static void MIX_AREAS2(unsigned int size,
 		       volatile signed int *sum, size_t dst_step,
 		       size_t src_step, size_t sum_step)
 {
+	unsigned int old_ebx;
+
 	/*
 	 *  ESI - src
 	 *  EDI - dst
@@ -248,13 +261,16 @@ static void MIX_AREAS2(unsigned int size,
 	__asm__ __volatile__ (
 		"\n"
 
-		"\tpush %%ebx\n"	/* ebx is GOT pointer (-fPIC) */
+		"\tmovl %%ebx, %7\n"	/* ebx is GOT pointer (-fPIC) */
 		/*
 		 *  initialization, load ESI, EDI, EBX registers
 		 */
 		"\tmovl %1, %%edi\n"
 		"\tmovl %2, %%esi\n"
 		"\tmovl %3, %%ebx\n"
+		"\tcmpl $0, %0\n"
+		"\tjnz 2f\n"
+		"\tjmp 6f\n"
 
 		"\t.p2align 4,,15\n"
 
@@ -327,10 +343,12 @@ static void MIX_AREAS2(unsigned int size,
 		"\tjmp 1b\n"
 		
 		"6:"
-		"\tpop %%ebx\n"		/* ebx is GOT pointer (-fPIC) */
+		"\tmovl %7, %%ebx\n"	/* ebx is GOT pointer (-fPIC) */
 
 		: /* no output regs */
-		: "m" (size), "m" (dst), "m" (src), "m" (sum), "m" (dst_step), "m" (src_step), "m" (sum_step)
+		: "m" (size), "m" (dst), "m" (src),
+		  "m" (sum), "m" (dst_step), "m" (src_step),
+		  "m" (sum_step), "m" (old_ebx)
 		: "esi", "edi", "edx", "ecx", "eax"
 	);
 }
