@@ -697,19 +697,19 @@ static void snd_pcm_ladspa_dump(snd_pcm_t *pcm, snd_output_t *out)
 }
 
 static snd_pcm_ops_t snd_pcm_ladspa_ops = {
-	close: snd_pcm_ladspa_close,
-	info: snd_pcm_plugin_info,
-	hw_refine: snd_pcm_ladspa_hw_refine,
-	hw_params: snd_pcm_ladspa_hw_params,
-	hw_free: snd_pcm_ladspa_hw_free,
-	sw_params: snd_pcm_plugin_sw_params,
-	channel_info: snd_pcm_plugin_channel_info,
-	dump: snd_pcm_ladspa_dump,
-	nonblock: snd_pcm_plugin_nonblock,
-	async: snd_pcm_plugin_async,
-	poll_revents: snd_pcm_plugin_poll_revents,
-	mmap: snd_pcm_plugin_mmap,
-	munmap: snd_pcm_plugin_munmap,
+	.close = snd_pcm_ladspa_close,
+	.info = snd_pcm_plugin_info,
+	.hw_refine = snd_pcm_ladspa_hw_refine,
+	.hw_params = snd_pcm_ladspa_hw_params,
+	.hw_free = snd_pcm_ladspa_hw_free,
+	.sw_params = snd_pcm_plugin_sw_params,
+	.channel_info = snd_pcm_plugin_channel_info,
+	.dump = snd_pcm_ladspa_dump,
+	.nonblock = snd_pcm_plugin_nonblock,
+	.async = snd_pcm_plugin_async,
+	.poll_revents = snd_pcm_plugin_poll_revents,
+	.mmap = snd_pcm_plugin_mmap,
+	.munmap = snd_pcm_plugin_munmap,
 };
 
 static int snd_pcm_ladspa_check_file(snd_pcm_ladspa_plugin_t * const plugin,
@@ -771,6 +771,8 @@ static int snd_pcm_ladspa_check_dir(snd_pcm_ladspa_plugin_t * const plugin,
 		}
 		
 		filename = malloc(len + strlen(dirent->d_name) + 1 + need_slash);
+		if (filename == NULL)
+			return -ENOMEM;
 		strcpy(filename, path);
 		if (need_slash)
 			strcat(filename, "/");
@@ -880,6 +882,8 @@ static int snd_pcm_ladspa_parse_ioconfig(snd_pcm_ladspa_plugin_t *lplug,
 		}
 		if (count > 0) {
 			array = (unsigned int *)calloc(count, sizeof(unsigned int));
+			if (! array)
+				return -ENOMEM;
 			memset(array, 0xff, count * sizeof(unsigned int));
 			io->port_bindings_size = count;
 			io->port_bindings = array;
@@ -928,6 +932,8 @@ static int snd_pcm_ladspa_parse_ioconfig(snd_pcm_ladspa_plugin_t *lplug,
 			if ((lplug->desc->PortDescriptors[idx] & (io->pdesc | LADSPA_PORT_CONTROL)) == (io->pdesc | LADSPA_PORT_CONTROL))
 				count++;
 		array = (LADSPA_Data *)calloc(count, sizeof(LADSPA_Data));
+		if (!array)
+			return -ENOMEM;
 		io->controls_size = count;
 		io->controls = array;
 		snd_config_for_each(i, next, controls) {
