@@ -79,6 +79,8 @@ typedef struct {
 	int close_slave;
 	snd_atomic_write_t watom;
 	snd_pcm_uframes_t appl_ptr, hw_ptr;
+	snd_pcm_uframes_t orig_avail_min;
+	snd_pcm_sw_params_t sw_params;
 	enum rate_type type;
 	unsigned int get_idx;
 	unsigned int put_idx;
@@ -556,10 +558,11 @@ static int snd_pcm_rate_sw_params(snd_pcm_t *pcm, snd_pcm_sw_params_t * params)
 {
 	snd_pcm_rate_t *rate = pcm->private_data;
 	snd_pcm_t *slave = rate->slave;
-	snd_pcm_sw_params_t sparams;
+	snd_pcm_sw_params_t *sparams;
 	snd_pcm_uframes_t boundary1, boundary2;
 
-	sparams = *params;
+	rate->sw_params = *params;
+	sparams = &rate->sw_params;
 	if ((rate->pitch >= LINEAR_DIV ? 1 : 0) ^ (pcm->stream == SND_PCM_STREAM_CAPTURE ? 1 : 0)) {
 		boundary1 = pcm->buffer_size;
 		boundary2 = slave->buffer_size;
@@ -576,7 +579,7 @@ static int snd_pcm_rate_sw_params(snd_pcm_t *pcm, snd_pcm_sw_params_t * params)
 		}
 	}
 	params->boundary = boundary1;
-	sparams.boundary = boundary2;
+	sparams->boundary = boundary2;
 	if (pcm->stream == SND_PCM_STREAM_PLAYBACK) {
 		rate->pitch = (((u_int64_t)slave->period_size * LINEAR_DIV) + pcm->period_size - 1) / pcm->period_size;
 		do {
@@ -624,17 +627,24 @@ static int snd_pcm_rate_sw_params(snd_pcm_t *pcm, snd_pcm_sw_params_t * params)
 		} while (1);
 		assert((snd_pcm_uframes_t)snd_pcm_rate_slave_frames(pcm, pcm->period_size - 1) == slave->period_size - 1);
 	}
-	recalc(pcm, &sparams.avail_min);
-	recalc(pcm, &sparams.xfer_align);
-	recalc(pcm, &sparams.start_threshold);
-	if (sparams.stop_threshold >= sparams.boundary) {
-		sparams.stop_threshold = sparams.boundary;
-	} else {
-		recalc(pcm, &sparams.stop_threshold);
+	recalc(pcm, &sparams->avail_min);
+	rate->orig_avail_min = sparams->avail_min;
+	recalc(pcm, &sparams->xfer_align);
+	recalc(pcm, &sparams->start_threshold);
+	if (sparams->start_threshold <= slave->buffer_size) {
+		if (sparams->start_threshold > (slave->buffer_size / sparams->avail_min) * sparams->avail_min)
+			sparams->start_threshold = (slave->buffer_size / sparams->avail_min) * sparams->avail_min;
+		if (sparams->start_threshold > (slave->buffer_size / sparams->xfer_align) * sparams->xfer_align)
+			sparams->start_threshold = (slave->buffer_size / sparams->xfer_align) * sparams->xfer_align;
 	}
-	recalc(pcm, &sparams.silence_threshold);
-	recalc(pcm, &sparams.silence_size);
-	return snd_pcm_sw_params(slave, &sparams);
+	if (sparams->stop_threshold >= sparams->boundary) {
+		sparams->stop_threshold = sparams->boundary;
+	} else {
+		recalc(pcm, &sparams->stop_threshold);
+	}
+	recalc(pcm, &sparams->silence_threshold);
+	recalc(pcm, &sparams->silence_size);
+	return snd_pcm_sw_params(slave, sparams);
 }
 
 static int snd_pcm_rate_init(snd_pcm_t *pcm)
@@ -913,6 +923,36 @@ static int snd_pcm_rate_resume(snd_pcm_t *pcm)
 	return snd_pcm_resume(rate->slave);
 }
 
+static int snd_pcm_rate_poll_ask(snd_pcm_t *pcm)
+{
+	snd_pcm_rate_t *rate = pcm->private_data;
+	snd_pcm_uframes_t avail_min;
+	int err;
+
+	if (rate->slave->fast_ops->poll_ask) {
+		err = rate->slave->fast_ops->poll_ask(rate->slave->fast_op_arg);
+		if (err < 0)
+			return err;
+	}
+	avail_min = rate->appl_ptr % pcm->period_size;
+	if (avail_min > 0) {
+		recalc(pcm, &avail_min);
+		if (avail_min < rate->slave->buffer_size &&
+		    avail_min != rate->slave->period_size)
+			avail_min++;	/* 1st small little rounding correction */
+		if (avail_min < rate->slave->buffer_size &&
+		    avail_min != rate->slave->period_size)
+			avail_min++;	/* 2nd small little rounding correction */
+		avail_min += rate->orig_avail_min;
+	} else {
+		avail_min = rate->orig_avail_min;
+	}
+	if (rate->sw_params.avail_min == avail_min)
+		return 0;
+	rate->sw_params.avail_min = avail_min;
+	return snd_pcm_sw_params(rate->slave, &rate->sw_params);
+}
+
 static int snd_pcm_rate_commit_next_period(snd_pcm_t *pcm, snd_pcm_uframes_t appl_offset)
 {
 	snd_pcm_rate_t *rate = pcm->private_data;
@@ -1045,6 +1085,8 @@ static int snd_pcm_rate_grab_next_period(snd_pcm_t *pcm, snd_pcm_uframes_t hw_of
 	      __partial:
 		xfer = 0;
 		cont = rate->slave->buffer_size - slave_offset;
+		if (cont > rate->slave->period_size)
+			cont = rate->slave->period_size;
 		snd_pcm_areas_copy(rate->sareas, 0,
 				   slave_areas, slave_offset,
 				   pcm->channels, cont,
@@ -1131,7 +1173,7 @@ static snd_pcm_sframes_t snd_pcm_rate_mmap_commit(snd_pcm_t *pcm,
 	appl_offset = (rate->appl_ptr - xfer) % pcm->buffer_size;
 	xfer = pcm->period_size - xfer;
 	if (xfer >= size) {
-		if (xfer == size) {
+		if (xfer == size && (snd_pcm_uframes_t)slave_size >= rate->slave->period_size) {
 			err = snd_pcm_rate_commit_next_period(pcm, appl_offset);
 			if (err < 0)
 				return err;
@@ -1143,17 +1185,20 @@ static snd_pcm_sframes_t snd_pcm_rate_mmap_commit(snd_pcm_t *pcm,
 		snd_atomic_write_end(&rate->watom);
 		return size;
 	} else {
-		size -= xfer;
-		err = snd_pcm_rate_commit_next_period(pcm, appl_offset);
-		if (err < 0)
-			return err;
-		if (err == 0)
-			return 0;
+		if ((snd_pcm_uframes_t)slave_size >= rate->slave->period_size) {
+			err = snd_pcm_rate_commit_next_period(pcm, appl_offset);
+			if (err < 0)
+				return err;
+			if (err == 0)
+				return 0;
+		}
 		snd_atomic_write_begin(&rate->watom);
 		snd_pcm_mmap_appl_forward(pcm, xfer);
 		snd_atomic_write_end(&rate->watom);
 		appl_offset += pcm->period_size;
 		appl_offset %= pcm->buffer_size;
+		size -= xfer;
+		slave_size -= rate->slave->period_size;
 	}
 	while ((snd_pcm_uframes_t)size >= pcm->period_size &&
 	       (snd_pcm_uframes_t)slave_size >= rate->slave->period_size) {
@@ -1291,6 +1336,7 @@ static snd_pcm_fast_ops_t snd_pcm_rate_fast_ops = {
 	.rewind = snd_pcm_rate_rewind,
 	.forward = snd_pcm_rate_forward,
 	.resume = snd_pcm_rate_resume,
+	.poll_ask = snd_pcm_rate_poll_ask,
 	.writei = snd_pcm_mmap_writei,
 	.writen = snd_pcm_mmap_writen,
 	.readi = snd_pcm_mmap_readi,
